@@ -1,21 +1,5 @@
-from typing import List, Tuple, Union
-
-from auto_editor.ffwrapper import FileInfo
-from auto_editor.utils.func import aspect_ratio
-
-
-def frames_to_timecode(frames: Union[int, float], fps: float) -> str:
-    seconds = frames / fps
-
-    m, _s = divmod(seconds, 60)
-    h, m = divmod(m, 60)
-
-    if len(str(int(_s))) == 1:
-        s = f"0{round(_s, 3):.3f}"
-    else:
-        s = f"{round(_s, 3):.3f}"
-
-    return f"{int(h):02d}:{int(m):02d}:{s}"
+from auto_editor.timeline import Timeline
+from auto_editor.utils.func import aspect_ratio, to_timecode
 
 
 def timecode_to_frames(timecode: str, fps: float) -> int:
@@ -27,13 +11,17 @@ def timecode_to_frames(timecode: str, fps: float) -> int:
 
 
 def shotcut_xml(
-    inp: FileInfo,
     output: str,
-    chunks: List[Tuple[int, int, float]],
+    timeline: Timeline,
 ) -> None:
-    width, height = inp.gwidth, inp.gheight
+    width, height = timeline.res
     num, den = aspect_ratio(width, height)
 
+    chunks = timeline.chunks
+    if chunks is None:
+        raise ValueError("Timeline too complex")
+    fps = timeline.fps
+    inp = timeline.inp
     global_out = inp.duration
 
     version = "21.05.18"
@@ -42,15 +30,14 @@ def shotcut_xml(
         out.write('<?xml version="1.0" standalone="no"?>\n')
         out.write(
             '<mlt LC_NUMERIC="C" version="7.1.0" '
-            + f'title="Shotcut version {version}" '
-            + 'producer="main_bin">\n'
+            f'title="Shotcut version {version}" producer="main_bin">\n'
         )
         out.write(
             '\t<profile description="automatic" '
-            + f'width="{width}" height="{height}" '
-            + 'progressive="1" sample_aspect_num="1" sample_aspect_den="1" '
-            + f'display_aspect_num="{num}" display_aspect_den="{den}" '
-            + f'frame_rate_num="{inp.gfps}" frame_rate_den="1" colorspace="709"/>\n'
+            f'width="{width}" height="{height}" '
+            'progressive="1" sample_aspect_num="1" sample_aspect_den="1" '
+            f'display_aspect_num="{num}" display_aspect_den="{den}" '
+            f'frame_rate_num="{fps}" frame_rate_den="1" colorspace="709"/>\n'
         )
         out.write('\t<playlist id="main_bin">\n')
         out.write('\t\t<property name="xml_retain">1</property>\n')
@@ -84,24 +71,21 @@ def shotcut_xml(
 
             speed = clip[2]
 
-            _out = frames_to_timecode(clip[1] / speed, inp.gfps)
-            length = frames_to_timecode((clip[1] / speed) + 1, inp.gfps)
+            _out = to_timecode(clip[1] / speed / fps, "standard")
+            length = to_timecode((clip[1] / speed + 1) / fps, "standard")
 
             if speed == 1:
                 resource = inp.path
                 caption = inp.basename
                 out.write(f'\t<chain id="chain{chains}" out="{_out}">\n')
-                chains += 1
             else:
                 resource = f"{speed}:{inp.path}"
                 caption = f"{inp.basename} ({speed}x)"
                 out.write(
-                    '\t<producer id="producer{}" in="00:00:00.000" out="{}">\n'.format(
-                        producers, _out
-                    )
+                    f'\t<producer id="producer{producers}" in="00:00:00.000" out="{_out}">\n'
                 )
                 producers += 1
-                chains += 1  # yes, Shotcut does this.
+            chains += 1
 
             out.write(f'\t\t<property name="length">{length}</property>\n')
             out.write('\t\t<property name="eof">pause</property>\n')
@@ -157,8 +141,8 @@ def shotcut_xml(
 
             out_len = max((clip[1] - 2) / speed, 0)
 
-            _in = frames_to_timecode(in_len, inp.gfps)
-            _out = frames_to_timecode(out_len, inp.gfps)
+            _in = to_timecode(in_len / fps, "standard")
+            _out = to_timecode(out_len / fps, "standard")
 
             tag_name = f"chain{i}"
             if speed != 1:
@@ -172,7 +156,7 @@ def shotcut_xml(
 
         out.write(
             f'\t<tractor id="tractor0" title="Shotcut version {version}" '
-            + f'in="00:00:00.000" out="{global_out}">\n'
+            f'in="00:00:00.000" out="{global_out}">\n'
         )
         out.write('\t\t<property name="shotcut">1</property>\n')
         out.write('\t\t<property name="shotcut:projectAudioChannels">2</property>\n')
