@@ -1,17 +1,15 @@
-# Internal Libraries
 import os.path
 import subprocess
+from fractions import Fraction
+from typing import Tuple
 
-# Typing
-from typing import List, Tuple
-
-# Included Libraries
-from auto_editor.utils.log import Log
-from auto_editor.ffwrapper import FFmpeg, FileInfo
-from auto_editor.utils.progressbar import ProgressBar
-
+from auto_editor.ffwrapper import FFmpeg
+from auto_editor.objects import EllipseObj, ImageObj, RectangleObj, TextObj
+from auto_editor.output import get_vcodec, video_quality
+from auto_editor.timeline import Timeline
 from auto_editor.utils.encoder import encoders
-from auto_editor.utils.video import get_vcodec, video_quality
+from auto_editor.utils.log import Log
+from auto_editor.utils.progressbar import ProgressBar
 
 # From: github.com/PyAV-Org/PyAV/blob/main/av/video/frame.pyx
 allowed_pix_fmt = {
@@ -90,12 +88,9 @@ def one_pos_two_pos(
 
 def render_av(
     ffmpeg: FFmpeg,
-    track: int,
-    inp: FileInfo,
+    timeline: Timeline,
     args,
-    chunks: List[Tuple[int, int, float]],
     progress: ProgressBar,
-    effects,
     rules,
     temp: str,
     log: Log,
@@ -107,26 +102,24 @@ def render_av(
     except ImportError:
         log.import_error("av")
     try:
-        from PIL import Image, ImageDraw, ImageFont, ImageChops
+        from PIL import Image, ImageChops, ImageDraw, ImageFont
     except ImportError:
         log.import_error("Pillow")
 
-    def set_static_assets(all_objects, log: Log):
-        # Save reloading the same thing over and over.
-
-        new_objects = []
-
-        for obj in all_objects:
-            if obj._type == "text":
+    font_cache = {}
+    img_cache = {}
+    for layer in timeline.v:
+        for obj in layer:
+            if isinstance(obj, TextObj) and obj.font not in font_cache:
                 try:
-                    obj.font = ImageFont.truetype(obj.font, obj.size)
+                    font_cache[obj.font] = ImageFont.truetype(obj.font, obj.size)
                 except OSError:
                     if obj.font == "default":
-                        obj.font = ImageFont.load_default()
+                        font_cache["default"] = ImageFont.load_default()
                     else:
                         log.error(f"Font '{obj.font}' not found.")
 
-            if obj._type == "image":
+            if isinstance(obj, ImageObj) and obj.src not in img_cache:
                 source = Image.open(obj.src)
                 source = source.convert("RGBA")
                 source = source.rotate(obj.rotate, expand=True)
@@ -136,76 +129,81 @@ def render_av(
                         "RGBA", source.size, (255, 255, 255, int(obj.opacity * 255))
                     ),
                 )
-                obj.src = source
+                img_cache[obj.src] = source
 
-            new_objects.append(obj)
-
-        return new_objects
-
-    def render_objects(sheet, all_objects, index: int, frame, pix_fmt: str):
+    def render_frame(frame, pix_fmt: str, timeline, index):
         img = frame.to_image().convert("RGBA")
 
-        for item in sheet[index]:
-            obj = all_objects[item]
+        for layer in timeline.v:
+            for obj in layer:
+                obj_img = Image.new("RGBA", img.size, (255, 255, 255, 0))
+                draw = ImageDraw.Draw(obj_img)
 
-            obj_img = Image.new("RGBA", img.size, (255, 255, 255, 0))
-            draw = ImageDraw.Draw(obj_img)
+                if isinstance(obj, TextObj):
+                    text_w, text_h = draw.textsize(
+                        obj.content, font=font_cache[obj.font]
+                    )
+                    pos = apply_anchor(obj.x, obj.y, text_w, text_h, "ce")
+                    draw.text(
+                        pos,
+                        obj.content,
+                        font=font_cache[obj.font],
+                        fill=obj.fill,
+                        align=obj.align,
+                        stroke_width=obj.stroke,
+                        stroke_fill=obj.strokecolor,
+                    )
 
-            if obj._type == "text":
-                text_w, text_h = draw.textsize(obj.content, font=obj.font)
-                pos = apply_anchor(obj.x, obj.y, text_w, text_h, "ce")
-                draw.text(
-                    pos,
-                    obj.content,
-                    font=obj.font,
-                    fill=obj.fill,
-                    align=obj.align,
-                    stroke_width=obj.stroke,
-                    stroke_fill=obj.strokecolor,
-                )
+                if isinstance(obj, RectangleObj):
+                    draw.rectangle(
+                        one_pos_two_pos(
+                            obj.x, obj.y, obj.width, obj.height, obj.anchor
+                        ),
+                        fill=obj.fill,
+                        width=obj.stroke,
+                        outline=obj.strokecolor,
+                    )
 
-            if obj._type == "rectangle":
-                draw.rectangle(
-                    one_pos_two_pos(obj.x, obj.y, obj.width, obj.height, obj.anchor),
-                    fill=obj.fill,
-                    width=obj.stroke,
-                    outline=obj.strokecolor,
-                )
+                if isinstance(obj, EllipseObj):
+                    draw.ellipse(
+                        one_pos_two_pos(
+                            obj.x, obj.y, obj.width, obj.height, obj.anchor
+                        ),
+                        fill=obj.fill,
+                        width=obj.stroke,
+                        outline=obj.strokecolor,
+                    )
 
-            if obj._type == "ellipse":
-                draw.ellipse(
-                    one_pos_two_pos(obj.x, obj.y, obj.width, obj.height, obj.anchor),
-                    fill=obj.fill,
-                    width=obj.stroke,
-                    outline=obj.strokecolor,
-                )
+                if isinstance(obj, ImageObj):
+                    img_w, img_h = img_cache[obj.src].size
+                    pos = apply_anchor(obj.x, obj.y, img_w, img_h, obj.anchor)
+                    obj_img.paste(img_cache[obj.src], pos)
 
-            if obj._type == "image":
-                img_w, img_h = obj.src.size
-                pos = apply_anchor(obj.x, obj.y, img_w, img_h, obj.anchor)
-                obj_img.paste(obj.src, pos)
-
-            img = Image.alpha_composite(img, obj_img)
+                img = Image.alpha_composite(img, obj_img)
 
         return frame.from_image(img).reformat(format=pix_fmt)
 
+    chunks = timeline.chunks
+
+    if chunks is None:
+        log.error("Timeline too complex")
+
+    inp = timeline.inp
+
     if chunks[-1][2] == 99999:
         chunks.pop()
 
-    total_frames = chunks[-1][1]
-    progress.start(total_frames, "Creating new video")
+    progress.start(chunks[-1][1], "Creating new video")
 
-    fps = inp.videos[track].fps
-
-    container = av.open(inp.path, "r")
-    pix_fmt = container.streams.video[track].pix_fmt
+    cn = av.open(inp.path, "r")
+    pix_fmt = cn.streams.video[0].pix_fmt
 
     target_pix_fmt = pix_fmt
 
     if not pix_fmt_allowed(pix_fmt):
         target_pix_fmt = "yuv420p"
 
-    my_codec = get_vcodec(args, inp, rules)
+    my_codec = get_vcodec(args.video_codec, inp, rules)
 
     apply_video_later = True
 
@@ -217,15 +215,13 @@ def render_av(
 
     log.debug(f"apply video quality settings now: {not apply_video_later}")
 
-    video_stream = container.streams.video[track]
-    video_stream.thread_type = "AUTO"
+    stream = cn.streams.video[0]
+    stream.thread_type = "AUTO"
 
-    width = video_stream.width
-    height = video_stream.height
+    width = stream.width
+    height = stream.height
 
-    effects.all = set_static_assets(effects.all, log)
-
-    spedup = os.path.join(temp, f"spedup{track}.mp4")
+    spedup = os.path.join(temp, "spedup0.mp4")
 
     cmd = [
         "-hide_banner",
@@ -239,7 +235,7 @@ def render_av(
         "-s",
         f"{width}*{height}",
         "-framerate",
-        f"{fps}",
+        f"{timeline.fps}",
         "-i",
         "-",
         "-pix_fmt",
@@ -256,26 +252,34 @@ def render_av(
     process2 = ffmpeg.Popen(
         cmd, stdin=subprocess.PIPE, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
     )
+    assert process2.stdin is not None
 
-    input_equavalent = 0.0
+    input_equavalent = Fraction(0, 1)
     output_equavalent = 0
     chunk = chunks.pop(0)
 
-    tou = int(video_stream.time_base.denominator / fps)
-    log.debug(f"Tou {tou}")
+    tou = int(stream.time_base.denominator / stream.average_rate)
+    log.debug(f"Tou: {tou}")
 
     seek = 10
     seek_frame = None
     frames_saved = 0
 
-    SEEK_COST = int(
-        fps * 5
-    )  # Keyframes are usually spread out every 5 seconds or less.
+    # Keyframes are usually spread out every 5 seconds or less.
+    if args.no_seek:
+        SEEK_COST = 4294967295
+    else:
+        SEEK_COST = int(stream.average_rate * 5)
     SEEK_RETRY = SEEK_COST // 2
 
+    # Converting between two different framerates is a lot like applying a speed.
+    fps_convert = Fraction(stream.average_rate, Fraction(timeline.fps))
+
     try:
-        for frame in container.decode(video_stream):
-            index = int(frame.time * fps)
+        for frame in cn.decode(stream):
+            # frame.time == frame.pts * stream.time_base
+            index = round(frame.time * timeline.fps)
+            index2 = round(frame.time * stream.average_rate)
 
             if frame.key_frame:
                 log.debug(f"Keyframe {index} {frame.pts}")
@@ -292,20 +296,17 @@ def render_av(
                     break
 
             if chunk[2] == 99999:
-                if chunk[1] - index > SEEK_COST and index > seek:
-                    seek = index + SEEK_RETRY
+                if chunk[1] - index2 > SEEK_COST and index2 > seek:
+                    seek = index2 + SEEK_RETRY
 
                     seek_frame = index
-                    container.seek(chunk[1] * tou, stream=video_stream)
+                    cn.seek(chunk[1] * tou, stream=stream)
             else:
-                input_equavalent += 1 / chunk[2]
+                input_equavalent += Fraction(1, Fraction(chunk[2]) * fps_convert)
 
             while input_equavalent > output_equavalent:
-                if index in effects.sheet:
-                    frame = render_objects(
-                        effects.sheet, effects.all, index, frame, target_pix_fmt
-                    )
-                elif pix_fmt != target_pix_fmt:
+                # frame = render_frame(frame, target_pix_fmt, timeline, index)
+                if pix_fmt != target_pix_fmt:
                     frame = frame.reformat(format=target_pix_fmt)
 
                 in_bytes = frame.to_ndarray().tobytes()
@@ -327,8 +328,8 @@ def render_av(
 
     # Unfortunately, scaling has to be a concrete step.
     if args.scale != 1:
-        sped_input = os.path.join(temp, f"spedup{track}.mp4")
-        spedup = os.path.join(temp, f"scale{track}.mp4")
+        sped_input = os.path.join(temp, "spedup0.mp4")
+        spedup = os.path.join(temp, "scale0.mp4")
 
         cmd = [
             "-i",
