@@ -1,11 +1,10 @@
 import re
+from typing import List, Literal, Sequence, Tuple, Union
 
-from typing import List, Tuple, Sequence, Union, Optional
-
-from .func import clean_list
+ChunkType = List[Tuple[int, int, float]]
 
 
-def split_num_str(val: Union[str, int]) -> Tuple[Union[int, float], str]:
+def split_num_str(val: Union[str, int]) -> Tuple[float, str]:
     if isinstance(val, int):
         return val, ""
     index = 0
@@ -14,17 +13,12 @@ def split_num_str(val: Union[str, int]) -> Tuple[Union[int, float], str]:
             break
         index += 1
     num, unit = val[:index], val[index:]
-    if "." in num:
-        try:
-            float(num)
-        except ValueError:
-            raise TypeError(f"Invalid number: '{val}'")
-        return float(num), unit
+
     try:
-        int(num)
+        float(num)
     except ValueError:
         raise TypeError(f"Invalid number: '{val}'")
-    return int(num), unit
+    return float(num), unit
 
 
 def unit_check(unit: str, allowed_units: Sequence[str]) -> None:
@@ -32,15 +26,15 @@ def unit_check(unit: str, allowed_units: Sequence[str]) -> None:
         raise TypeError(f"Unknown unit: '{unit}'")
 
 
-def float_type(val: Union[str, int, float]) -> float:
-    if isinstance(val, (int, float)):
-        return float(val)
+def float_type(val: Union[str, float]) -> float:
+    if not isinstance(val, str):
+        return val
 
     num, unit = split_num_str(val)
     unit_check(unit, ("%", ""))
     if unit == "%":
-        return float(num / 100)
-    return float(num)
+        return num / 100
+    return num
 
 
 def sample_rate_type(val: str) -> int:
@@ -68,42 +62,42 @@ def anchor_type(val: str) -> str:
 
 
 def margin_type(val: str) -> Tuple[Union[int, str], Union[int, str]]:
-    vals = val.split(",")
+    vals = val.strip().split(",")
     if len(vals) == 1:
         vals.append(vals[0])
     if len(vals) != 2:
-        raise TypeError("Too many comma arguments for margin_type")
+        raise TypeError("--margin has too many arguments.")
     return frame_type(vals[0]), frame_type(vals[1])
 
 
-def comma_type(
-    _val: str, min_args: int = 1, max_args: Optional[int] = None, name: str = ""
-) -> List[str]:
-    val = clean_list(_val.split(","), "\r\n\t")
-    if min_args > len(val):
-        raise TypeError(f"Too few comma arguments for {name}.")
-    if max_args is not None and len(val) > max_args:
-        raise TypeError(f"Too many comma arguments for {name}.")
-    return val
+def comma_type(name: str, val: str, min_args: int, max_args: int) -> List[str]:
+    vals = val.strip().split(",")
+    if min_args > len(vals):
+        raise TypeError(f"Too few arguments for {name}.")
+    if len(vals) > max_args:
+        raise TypeError(f"Too many arguments for {name}.")
+    return vals
 
 
 def range_type(val: str) -> List[str]:
-    return comma_type(val, 2, 2, "range_type")
+    return comma_type("range_type", val, 2, 2)
 
 
 def speed_range_type(val: str) -> List[str]:
-    return comma_type(val, 3, 3, "speed_range_type")
+    return comma_type("speed_range_type", val, 3, 3)
 
 
-def text_content(val: str) -> str:
-    return val.replace("\\n", "\n")
+AlignType = Literal["left", "center", "right"]
 
 
-def align_type(val: str) -> str:
-    allowed = ("left", "right", "center")
-    if val not in allowed:
-        raise TypeError("Align must be: " + " ".join(allowed))
-    return val
+def align_type(val: str) -> AlignType:
+    if val == "left":
+        return "left"
+    if val == "center":
+        return "center"
+    if val == "right":
+        return "right"
+    raise TypeError("Align must be 'left', 'right', or 'center'")
 
 
 def color_type(val: str) -> str:
@@ -130,6 +124,15 @@ def color_type(val: str) -> str:
     raise ValueError(f"Invalid Color: '{color}'")
 
 
+StreamType = Union[int, Literal["all"]]
+
+
+def stream_type(val: str) -> StreamType:
+    if val == "all":
+        return "all"
+    return int(val)
+
+
 colormap = {
     # Taken from https://www.w3.org/TR/css-color-4/#named-color
     "aliceblue": "#f0f8ff",
